@@ -296,10 +296,6 @@ void Disc::setEncoding( bool b )
 	}
 	else
 		progressBar->hide();
-	if ( para ) {
-		delete para;
-		para = NULL;
-	}
 }
 
 void Disc::encode()
@@ -321,22 +317,24 @@ void Disc::encode()
 		KMessageBox::information( mainWidget, i18n("You must select the tracks to rip."), i18n("Warning") );
 		return;
 	}
-	if ( !para ) {
-		if ( trackCurrent )
-			emit stop();
-		para = new Paranoia();
-		if ( !para->init( currentDevice ) ) {
-			KMessageBox::information( mainWidget, i18n("Unable to initialize cdparanoia."), i18n("Warning") );
-			delete para;
-			para = NULL;
-			return;
-		}
+	setEncoding( true );
+	if ( trackCurrent )
+		emit stop();
+	para = new Paranoia();
+	if ( !para->init( currentDevice ) ) {
+		KMessageBox::information( mainWidget, i18n("Unable to initialize cdparanoia."), i18n("Warning") );
+		delete para;
+		para = NULL;
+		setEncoding( false );
+		return;
 	}
-	if ( para ) {
-		if ( para->encode( tracklist, mainWidget ) ) {
-			setEncoding( true );
-			encodeTimer.start( 1000 );
-		}
+
+	if ( para->encode( tracklist, mainWidget ) )
+		encodeTimer.start( 1000 );
+	else {
+		delete para;
+		para = NULL;
+		setEncoding( false );
 	}
 }
 
@@ -344,8 +342,13 @@ void Disc::encodeProgress()
 {
 	if ( para->running() )
 		progressBar->setProgress( para->getProgress() );
-	else
+	else {
+		encodeTimer.stop();
+		delete para;
+		para = NULL;
 		setEncoding( false );
+	}
+
 }
 
 void Disc::setCurrent( int n )
@@ -439,8 +442,6 @@ void Disc::startCD( const QString &device, bool rip )
 	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
 	qApp->processEvents();
 
-	if ( para )
-		delete para;
 	para = new Paranoia();
 	for ( i=0; i<(int)s.count(); i++ ) {
 		if ( (init = para->init( s[i] )) ) {
@@ -511,17 +512,15 @@ void Disc::startCD( const QString &device, bool rip )
 			mrl.setMime( QString("audio/cdda") );
 			trackCurrent = mrl.track().toInt();
 			setCurrent( trackCurrent );
-			delete para;
-			para = NULL;
 			emit play( mrl, this );
 		}
 	}
 	else {
 		QApplication::restoreOverrideCursor();
 		KMessageBox::information( 0, i18n("No audio CD found."), i18n("Warning") );
-		delete para;
-		para = NULL;
 	}
+	delete para;
+	para = NULL;
 }
 
 void Disc::startDVD( const QString &device )
