@@ -29,6 +29,8 @@
 
 #include <linux/dvb/frontend.h>
 
+#include "kaffeinedvbevents.h"
+
 #define MAXAPID 20
 #define MAXSUBPID 10
 
@@ -42,7 +44,6 @@ public:
 	QString channel;
 	QDateTime begin;
 	QTime duration;
-	int filetype;
 	char running;
 	int mode;
 };
@@ -89,6 +90,7 @@ public:
 	QValueList<unsigned long> freqlist;
 	char pol;
 	unsigned long sr;
+	unsigned short nid;
 	unsigned short tsid;
 	fe_spectral_inversion_t inversion;
 	fe_modulation_t modulation;
@@ -132,43 +134,4 @@ public:
 	QPixmap pix;
 };
 
-class ShortEvent
-{
-
-public:
-
-	ShortEvent();
-	~ShortEvent();
-
-	QString name;
-	QString text;
-};
-
-class EventDesc
-{
-
-public:
-
-	EventDesc();
-	~EventDesc();
-	void printOut();
-
-	QString source;
-	unsigned char tid;
-	unsigned short sid;
-	unsigned short tsid;
-	unsigned short nid;
-	unsigned char sn;
-	unsigned char lsn;
-	unsigned short eid;
-	unsigned char running;
-	QDateTime startDateTime;
-	QTime duration;
-	QPtrList<ShortEvent> shortEvents;
-	QPtrList<QString> extEvents;
-	QString title;
-	QString subtitle;
-	unsigned int loop;
-};
-
 #endif /* CHANNELDESC_H */
