@@ -38,6 +38,7 @@
 
 #include "dvbconfig.h"
 #include "kaffeinedvbplugin.h"
+#include "dvbpanel.h"
 
 
 
@@ -72,6 +73,21 @@ void MPushButton::isClicked()
 
 
 
+MCAMButton::MCAMButton( QWidget *parent, int devNum ) : QPushButton( i18n("CAM"), parent )
+{
+	deviceNumber = devNum;
+	connect( this, SIGNAL(clicked()), this, SLOT(isClicked()) );
+}
+
+
+
+void MCAMButton::isClicked()
+{
+	emit clicked( deviceNumber );
+}
+
+
+
 MComboBox::MComboBox( QWidget *parent, int devNum, int lnbNum ) : QComboBox( parent )
 {
 	deviceNumber = devNum;
@@ -109,6 +125,8 @@ Device::Device( int anum, int tnum, fe_type_t t, const QString &n, bool as )
 	source = "";
 	canAutoscan= as;
 	tuningTimeout = 1500;
+	hasCAM = false;
+	camMaxService = 1;
 }
 
 
@@ -292,9 +310,9 @@ bool DVBconfig::localData()
 
 bool DVBconfig::haveData()
 {
-	if ( !QDir( dvbConfigDir+"dvb-s" ).exists() || !QDir( dvbConfigDir+"dvb-c" ).exists() || !QDir( dvbConfigDir+"dvb-t" ).exists() ) {
+	if ( !QDir( dvbConfigDir+"dvb-s" ).exists() || !QDir( dvbConfigDir+"dvb-c" ).exists() || !QDir( dvbConfigDir+"dvb-t" ).exists() || !QDir( dvbConfigDir+"atsc" ).exists()) {
 		loadDvbData(0);
-		if ( !QDir( dvbConfigDir+"dvb-s" ).exists() || !QDir( dvbConfigDir+"dvb-c" ).exists() || !QDir( dvbConfigDir+"dvb-t" ).exists() ) {
+		if ( !QDir( dvbConfigDir+"dvb-s" ).exists() || !QDir( dvbConfigDir+"dvb-c" ).exists() || !QDir( dvbConfigDir+"dvb-t" ).exists() || !QDir( dvbConfigDir+"atsc" ).exists() ) {
 			if ( !localData() )
 				return false;
 		}
@@ -313,6 +331,7 @@ QStringList DVBconfig::getSourcesList( fe_type_t type )
 		case FE_QPSK : s = "dvb-s"; break;
 		case FE_QAM : s = "dvb-c"; break;
 		case FE_OFDM : s = "dvb-t"; break;
+		case FE_ATSC : s = "atsc"; break;
 		default : return list;
 	}
 	list = QDir( dvbConfigDir+s ).entryList( QDir::Files, QDir::Name );
@@ -426,6 +445,7 @@ void DVBconfig::readConfig()
 			devList.at(i)->lnb[j].speed13v = config->readDoubleNumEntry( QString("DVB%1_LNB%2_speed13v").arg(i).arg(j), 2.5 );
 			devList.at(i)->lnb[j].speed18v = config->readDoubleNumEntry( QString("DVB%1_LNB%2_speed18v").arg(i).arg(j), 1.5 );
 		}
+		devList.at(i)->camMaxService = config->readNumEntry( QString("DVB%1_CAM_MAX").arg(i), 1 );
 	}
 	j = config->readNumEntry( "NumCategories", 0 );
 	for ( i=0; i<j; i++ )
@@ -479,6 +499,7 @@ void DVBconfig::saveConfig()
 			config->writeEntry( QString("DVB%1_LNB%2_speed13v").arg(i).arg(j), devList.at(i)->lnb[j].speed13v );
 			config->writeEntry( QString("DVB%1_LNB%2_speed18v").arg(i).arg(j), devList.at(i)->lnb[j].speed18v );
 		}
+		config->writeEntry( QString("DVB%1_CAM_MAX").arg(i), devList.at(i)->camMaxService );
 	}
 	config->writeEntry( "NumCategories", categories.count() );
 	for ( i=0; i<(int)categories.count(); i++ ) {
@@ -511,7 +532,7 @@ bool DVBconfig::firstRun()
 
 
 
-DvbConfigDialog::DvbConfigDialog( DVBconfig *dc, QWidget *parent, KaffeineDvbPlugin *p ) :
+DvbConfigDialog::DvbConfigDialog( DvbPanel *pan, DVBconfig *dc, QWidget *parent, KaffeineDvbPlugin *p ) :
 	KDialogBase ( IconList, i18n("DVB Settings"), Ok|Cancel, Ok, parent, "dvbConfigDialog", true, true )
 {
 	QLabel *lab;
@@ -558,7 +579,14 @@ DvbConfigDialog::DvbConfigDialog( DVBconfig *dc, QWidget *parent, KaffeineDvbPlu
 			case FE_ATSC : dvbType->setText( i18n("Atsc") ); break;
 			default : dvbType->setText( i18n("Unknown") );
 		}
-		grid->addMultiCellWidget( dvbType, gridLine, gridLine, 1, 3 );
+		if ( dvbConfig->devList.at(i)->hasCAM ) {
+			grid->addWidget( dvbType, gridLine, 1 );
+			MCAMButton *camb = new MCAMButton( gb, i );
+			connect( camb, SIGNAL(clicked(int)), pan, SLOT(camClicked(int)) );
+			grid->addWidget( camb, gridLine, 2 );
+		}
+		else
+			grid->addMultiCellWidget( dvbType, gridLine, gridLine, 1, 3 );
 		++gridLine;
 
 		lab = new QLabel( i18n("Tuner timeout :"), gb );
