@@ -45,7 +45,6 @@
 #include <kxmlguifactory.h>
 #include <kparts/componentfactory.h>
 
-#include "kaffeineinput.h"
 #include "kaffeinedvbplugin.h"
 #include "dvbpanel.h"
 #include "channeldesc.h"
@@ -1091,7 +1090,7 @@ void DvbPanel::setConfig()
 
 void DvbPanel::showConfigDialog()
 {
-	int ret;
+	int i, ret;
 
 loop:
 	if ( !dvbConfig->haveData() ) {
@@ -1104,7 +1103,9 @@ loop:
 		return;
 	}
 
-	DvbConfigDialog dlg( dvbConfig, mainWidget, plug );
+	for ( i=0; i<(int)dvb.count(); i++ )
+		dvb.at(i)->probeCam();
+	DvbConfigDialog dlg( this, dvbConfig, mainWidget, plug );
 	connect( dlg.dumpBtn, SIGNAL(clicked()), this, SLOT(dumpEvents()) );
 	ret = dlg.exec();
 	disconnect( dlg.dumpBtn, SIGNAL(clicked()), this, SLOT(dumpEvents()) );
@@ -1116,6 +1117,13 @@ loop:
 
 
 
+void DvbPanel::camClicked( int devNum )
+{
+	dvb.at(devNum)->showCamDialog();
+}
+
+
+
 QPtrList<Transponder> DvbPanel::getSourcesStatus()
 {
 	int i, j;
@@ -1176,7 +1184,7 @@ void DvbPanel::fillChannelList( ChannelDesc *ch )
 		}
 		else if ( currentCategory!="All" && chan->category!=currentCategory )
 			continue;
-		it = new KListViewItem( channelsCb, QString().sprintf("%04d", chan->num), chan->name, chan->tp.source );
+		it = new KListViewItem( channelsCb, QString().sprintf("%05d", chan->num), chan->name, chan->tp.source );
 		if ( ch && ch==chan )
 			visible = it;
 		it->setDragEnabled( true );
@@ -1880,7 +1888,7 @@ void DvbPanel::next()
 
 	QListViewItem* nextItem;
 
-	QListViewItem* playingItem = channelsCb->findItem( QString().sprintf("%04d", dvbConfig->lastChannel), 0 );
+	QListViewItem* playingItem = channelsCb->findItem( QString().sprintf("%05d", dvbConfig->lastChannel), 0 );
 
 	if ( !playingItem == 0 ) // yes, it's in the current category
 	{
@@ -1906,7 +1914,7 @@ void DvbPanel::previous()
 
 	QListViewItem* prevItem;
 
-	QListViewItem* playingItem = channelsCb->findItem( QString().sprintf("%04d", dvbConfig->lastChannel), 0 );
+	QListViewItem* playingItem = channelsCb->findItem( QString().sprintf("%05d", dvbConfig->lastChannel), 0 );
 
 	if ( !playingItem == 0 ) // yes, it's in the current category
 	{
@@ -2363,7 +2371,7 @@ bool DvbPanel::getChannelList()
 			}
 
 			chan->pix.load( dvbConfig->dvbConfigIconsDir+chan->name );
-			it = new KListViewItem( channelsCb, QString().sprintf("%04d", chan->num), chan->name, chan->tp.source );
+			it = new KListViewItem( channelsCb, QString().sprintf("%05d", chan->num), chan->name, chan->tp.source );
 			it->setDragEnabled( true );
 			if ( !chan->pix.isNull() )
 				it->setPixmap( 1, chan->pix );
