@@ -2,6 +2,7 @@
  * dvbsi.cpp
  *
  * Copyright (C) 2003-2007 Christophe Thommeret <hftom@free.fr>
+ * Copyright (C) 2008 Devin Heitmueller <devin.heitmueller@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +31,21 @@
 
 #define TIMER_EVENT_SCAN_END 100
 
+// These values are taken from ATSC A/65C Sec 5
+#define PSIP_BASE_PID 0x1ffb
 
+// This shouldn't be necessary, but at least the HVR-950 doesn't seem to
+// find them in the time specified in the spec (needs more investigation)
+#define CYCLE_TIME_FUDGEFACTOR 1000
+
+// These values are taken from ATSC A/65C Sec 7.1
+// (all values in ms)
+#define PSIP_MAX_CYCLE_TIME_MGT 150 + CYCLE_TIME_FUDGEFACTOR
+#define PSIP_MAX_CYCLE_TIME_VCT 400 + CYCLE_TIME_FUDGEFACTOR
+
+// These values are taken from ATSC A/65C Sec 4.1
+#define PSIP_TABLE_TYPE_MGT     0xc7
+#define PSIP_TABLE_TYPE_TVCT    0xc8
 
 NitSection::NitSection( QPtrList<Transponder> *tp, bool *end, bool *ok, int anum, int tnum ) : KaffeineDVBsection( anum, tnum )
 {
@@ -456,6 +471,7 @@ bool DVBsi::tablePMT( unsigned char* buf )
 
 	while ( length>4 ) {
 		audio=ac3=false;
+		lang="";
 		type = getBits(buf,0,8);
 		pid = getBits(buf,11,13);
 		if ( type==1/*mpeg1*/ || type==2/*mpeg2*/ || type==16/*mpeg4*/ || type==27/*h264*/ ) {
@@ -466,6 +482,11 @@ bool DVBsi::tablePMT( unsigned char* buf )
 		if ( type==3 || type==4 ) {
 			audio = true;
 		}
+		if (type == 0x81) {
+		  // AC3 was added in ATSC A/52B (See A3.1 "Stream Type")
+		  audio = true;
+		  ac3 = true;
+		}
 		loop = getBits(buf,28,12);
 		buf +=5;
 		length -=(5+loop);
@@ -499,6 +520,7 @@ bool DVBsi::tablePMT( unsigned char* buf )
 					}
 					break;
 				case 0x6a :
+				case 0x81 :
 					audio = true;
 					ac3 = true;
 					break;
@@ -637,7 +659,204 @@ bool DVBsi::getSection( int pid, int tid, int timeout, int sid )
 	return true;
 }
 
+bool DVBsi::parseMGT( int pid, int tid, int timeout, int sid )
+{
+	unsigned char buf[4096];
+	int n=0;
+	int skip=0;
+	bool vct_found = false;
 
+	fprintf(stderr, "parseMGT called for 0x%02x 0x%02x\n", pid, tid);
+	if ( !setFilter( pid, tid, timeout ) )
+		return false;
+
+	if ( poll(pf,1,timeout)>0 ){
+		if ( pf[0].revents & POLLIN ){
+			n = read( fdDemux, buf, 4096 );
+			skip = 0;
+		}
+		else
+			skip++;
+	}
+	else
+		skip++;
+
+	if ( skip || n<4 ) {
+		fprintf(stderr,"\nInvalid section length or timeout: pid=%d\n\n", pid);
+		stopFilter();
+		return false;
+	}
+
+	// Parse the Master Guide Table Section
+	unsigned int protocol_version = getBits(buf,64,8);
+	unsigned int tables_defined = getBits(buf,72,16);
+	fprintf(stderr, "protocol_version = %d\n", protocol_version);
+	fprintf(stderr, "tables_defined = %d\n", tables_defined);
+
+	// Now let's go through the table entries....
+	unsigned char *t_entry = &buf[88/8];
+	for (unsigned int t = 0; t < tables_defined; t++) {
+		unsigned int table_type;
+		unsigned int table_pid;
+		unsigned int table_version_number;
+		unsigned int table_number_bytes;
+		unsigned int table_number_des_length;
+		table_type = getBits(t_entry, 0, 16);
+		table_pid = getBits(t_entry, 19, 13);
+		table_version_number = getBits(t_entry, 35, 5);
+		table_number_bytes = getBits(t_entry, 40, 32);
+		table_number_des_length = getBits(t_entry, 76, 12);
+		fprintf(stderr,
+			"MGT entry type=0x%04x pid=0x%04x ver=%d sz=%d\n",
+			table_type, table_pid, table_version_number,
+			table_number_bytes);
+
+		if (table_type == 0x0000 || table_type == 0x0001) {
+			// TVCT table found
+			vct_table = table_type;
+			vct_found = true;
+		} else if (table_type == 0x0002 ||
+			   table_type == 0x0003) {
+			// CVCT table found
+			vct_table = table_type;
+			vct_found = true;
+		}
+		t_entry += (11 + table_number_des_length);
+	}
+
+	stopFilter();
+	return vct_found;
+}
+
+bool DVBsi::parseVCT( int pid, int tid, int timeout, int sid )
+{
+	unsigned char buf[4096];
+	int n=0;
+	int skip=0;
+
+	fprintf(stderr, "parseVCT called for 0x%02x 0x%02x\n", pid, tid);
+	if ( !setFilter( pid, tid, timeout ) )
+		return false;
+
+	if ( poll(pf,1,timeout)>0 ){
+		if ( pf[0].revents & POLLIN ){
+			n = read( fdDemux, buf, 4096 );
+			skip = 0;
+		}
+		else
+			skip++;
+	}
+	else
+		skip++;
+
+	if ( skip || n<4 ) {
+		fprintf(stderr,"\nInvalid section length or timeout: pid=%d\n\n", pid);
+		stopFilter();
+		return false;
+	}
+
+	// Parse the Virtual Channel Table Section
+	unsigned int protocol_version = getBits(buf,64,8);
+	unsigned int num_channels = getBits(buf,72,8);
+	fprintf(stderr, "protocol_version = %d\n", protocol_version);
+	fprintf(stderr, "num_channels = %d\n", num_channels);
+
+	// Now let's go through the table entries....
+	unsigned char *t_entry = &buf[80/8];
+	for (unsigned int t = 0; t < num_channels; t++) {
+		char short_name[8];
+		unsigned int major_channel_num;
+		unsigned int minor_channel_num;
+		unsigned int modulation_mode;
+		unsigned int channel_tsid;
+		unsigned int program_number;
+		unsigned int access_controlled;
+		unsigned int hidden;
+		unsigned int service_type;
+		unsigned int source_id;
+		unsigned int reserved;
+		unsigned int descriptors_length;
+
+		// Short name
+		// Yes, I need a real UCS-2 to UTF-8 conversion here...
+		memset(short_name, 0, sizeof(short_name));
+		snprintf(short_name, sizeof(short_name), "%c%c%c%c%c%c%c",
+			 t_entry[1],t_entry[3],t_entry[5],t_entry[7],
+			 t_entry[9],t_entry[11],t_entry[13]);
+		reserved = getBits(t_entry, 112, 4);
+		major_channel_num = getBits(t_entry, 116, 10);
+		minor_channel_num = getBits(t_entry, 126, 10);
+		modulation_mode = getBits(t_entry, 136, 8);
+		channel_tsid = getBits(t_entry, 176, 16);
+		program_number = getBits(t_entry, 192, 16);
+		access_controlled = getBits(t_entry, 210, 1);
+		hidden = getBits(t_entry, 211, 1);
+		service_type = getBits(t_entry, 218, 6);
+		source_id = getBits(t_entry, 224, 16);
+		reserved = getBits(t_entry, 240, 6);
+		descriptors_length = getBits(t_entry, 246, 10);
+
+		fprintf(stderr, "short name=%s\n", short_name);
+		fprintf(stderr, "reserved=0x%04x\n", reserved);
+		fprintf(stderr, "major=%d\n", major_channel_num);
+		fprintf(stderr, "minor=%d\n", minor_channel_num);
+		fprintf(stderr, "modulation mode=0x%02x\n", modulation_mode);
+		fprintf(stderr, "channel_tsid=0x%04x\n", channel_tsid);
+		fprintf(stderr, "program_number=0x%04x\n", program_number);
+		fprintf(stderr, "access_controlled=0x%01x\n", access_controlled);
+		fprintf(stderr, "hidden=0x%01x\n", hidden);
+		fprintf(stderr, "service_type=0x%02x\n", service_type);
+		fprintf(stderr, "source_id=0x%04x\n", source_id);
+		fprintf(stderr, "des length=%d\n", descriptors_length);
+
+		ChannelDesc *desc = new ChannelDesc();
+		desc->tp.tsid = channel_tsid;
+		desc->name = QString("%1-%2 %3").arg(major_channel_num).arg(minor_channel_num).arg(short_name);
+		desc->sid = program_number;
+		if (access_controlled == 1)
+			desc->fta = 1;
+		else
+			desc->fta = 0; // 0 for free
+
+		// Algorithm taken from ATSC A/65C Sec 4.2
+		// However, the algorithm doesn't appear correct, as it
+		// truncates out data.  For example, both 68-1 and 4-1 would
+		// yield the same one_part_number
+		desc->num = (major_channel_num << 10) + minor_channel_num;
+		fprintf(stderr, "channel num=%d\n", desc->num);
+
+		// ATSC A/65C Sec 6.3.1 Table 6.7
+		if (service_type == 0x01) {
+			// Analog television (not supported)
+		} else if (service_type == 0x02) {
+			// ATSC Video
+			desc->type = 1;
+		} else if (service_type == 0x03) {
+			// ATSC Audio
+			desc->type = 2;
+		} else if (service_type == 0x02) {
+			// ATSC Data only service (not supported)
+		} else if (service_type > 0x05 && service_type < 0x3f) {
+			// Reserved (not supported)
+		} else {
+			// Unknown
+		}
+
+		// Now add the new channel to the list (if supported)
+		if ((desc->type == 1 || desc->type == 2) && hidden == 0) {
+			channels.append( desc );
+		} else {
+			fprintf(stderr, "Not adding channel\n");
+			delete desc;
+		}
+
+		// Advance to the next entry
+		t_entry += (32 + descriptors_length);
+	}
+
+	stopFilter();
+	return true;
+}
 
 void DVBsi::stop()
 {
@@ -701,7 +920,24 @@ void DVBsi::timerEvent( QTimerEvent *e )
 	}
 }
 
+// See ATSC standard A/65c for info on PSIP and what all these acronyms are...
+bool DVBsi::handle_atsc_transponder() {
+	// Loop through the MGT to get the list of PIDS for virtual channels
+	if (parseMGT(PSIP_BASE_PID, PSIP_TABLE_TYPE_MGT,
+		     PSIP_MAX_CYCLE_TIME_MGT) == false) {
+		// We couldn't find the MGT
+		fprintf(stderr, "Could not find MGT in stream.  Cannot continue\n");
+		return false;
+	};
 
+	// Now look at the TVCT for info on the individual channels found
+	if (parseVCT(PSIP_BASE_PID, PSIP_TABLE_TYPE_TVCT,
+		     PSIP_MAX_CYCLE_TIME_VCT) == false) {
+		fprintf(stderr, "Could not parse VCT in stream.  Cannot continue\n");
+		return false;
+	}
+	return true;
+}
 
 void DVBsi::run()
 {
@@ -744,12 +980,21 @@ void DVBsi::run()
 
 			indexChannels = j;
 			fprintf(stderr,"Transponders: %d/%d\n", i+1, transponders.count() );
+			fprintf(stderr,"scanMode=%d\n", scanMode);
 			if ( scanMode ) {
 				nitEnded = false;
 				ns = new NitSection( &transponders, &nitEnded, &ok, adapter, tuner ); //NIT
 			}
-			if ( !getSection( 0x11, 0x42 ) )         //SDT
-				continue;
+			if ( chan.tp.type == FE_ATSC ) {
+				// Separate out the ATSC scanning so that we
+				// don't interfere with existing DVB support
+				if (!handle_atsc_transponder())
+					continue;
+			}
+			else {
+				printf("it's dvb %d!\n", chan.tp.type);
+				getSection( 0x11, 0x42 );       //SDT
+			}
 			if ( !isRunning ) {
 				out();
 				return;
@@ -822,7 +1067,7 @@ void DVBsi::run()
 }
 
 
-
+// !!!! only there for debugging !!!!!!!!!!
 bool DVBsi::listChannels()
 {
 	QString s,t;
