@@ -66,6 +66,11 @@ public:
 	bool tablePMT( unsigned char* buf );
 	void serviceDesc( unsigned char* buf, ChannelDesc *desc );
 
+	// ATSC related methods
+	virtual bool handle_atsc_transponder();
+	virtual bool parseMGT( int pid, int tid, int timeout=5000, int sid=0 );
+	virtual bool parseVCT( int pid, int tid, int timeout=5000, int sid=0 );
+
 	QPtrList<ChannelDesc> channels;
 	QPtrList<Transponder> transponders;
 	DvbStream *dvb;
@@ -90,6 +95,9 @@ private:
 	int scanMode;
 	NitSection *ns;
 
+	/* ATSC related */
+	int vct_table;
+
 signals:
 
 	void end( bool );
