@@ -43,6 +43,7 @@
 
 #include <klocale.h>
 #include <kapplication.h>
+#include <kmessagebox.h>
 
 #include "dvbstream.h"
 #include "dvbevents.h"
@@ -70,8 +71,14 @@ DvbStream::DvbStream( Device *d, const QString &charset, EventTable *et )
 
 	out.setAutoDelete( true );
 
-	bool bok;
-	dvbEvents = new DVBevents( &bok, dvbDevice->adapter, dvbDevice->tuner, charset, et );
+	QString deviceType="NONE";
+	switch ( dvbDevice->type ) {
+		case FE_QPSK: deviceType = "DVBS"; break;
+		case FE_OFDM: deviceType = "DVBT"; break;
+		case FE_QAM: deviceType = "DVBC"; break;
+		case FE_ATSC: deviceType = "DVBA"; break;
+	}
+	dvbEvents = new DVBevents( deviceType, dvbDevice->adapter, dvbDevice->tuner, charset, et );
 
 	camProbed = false;
 	cam = NULL;
@@ -94,13 +101,23 @@ void DvbStream::probeCam()
 	int ci_type=0;
 	if ( camProbed )
 		return;
-	if ( (ci_type=DvbCam::probe( dvbDevice->adapter, 0 ))>0 )
-		cam = new DvbCam( dvbDevice->adapter, 0, dvbDevice->tuner, ci_type );
+	if ( (ci_type=DvbCam::probe( dvbDevice->adapter, 0 ))>0 ) {
+		cam = new DvbCam( dvbDevice->adapter, 0, dvbDevice->tuner, ci_type, dvbDevice->camMaxService );
+		dvbDevice->hasCAM = true;
+	}
 	camProbed = true;
 }
 
 
 
+void DvbStream::showCamDialog()
+{
+	if ( cam )
+		dvbDevice->camMaxService = cam->showCamDialog();
+}
+
+
+
 QStringList DvbStream::getSources( bool all )
 {
 	if ( !all ) {
@@ -281,6 +298,13 @@ Transponder DvbStream::getCurrentTransponder()
 
 
 
+unsigned long DvbStream::getCurrentFreq()
+{
+	return currentTransponder.freq;
+}
+
+
+
 bool DvbStream::isTuned()
 {
 	if ( fdFrontend )
@@ -480,6 +504,7 @@ int DvbStream::setDiseqc( int switchPos, ChannelDesc *chan, int hiband, int &rot
 		{ { 0xe0, 0x10, 0x38, 0xff, 0x00, 0x00 }, 4 },
 	};
 
+	int i;
 	int voltage18 = ( (chan->tp.pol=='H')||(chan->tp.pol=='h') );
 	int ci = 4 * switchPos + 2 * hiband + (voltage18 ? 1 : 0);
 
@@ -489,12 +514,16 @@ int DvbStream::setDiseqc( int switchPos, ChannelDesc *chan, int hiband, int &rot
 
 	if ( ioctl(fdFrontend, FE_SET_TONE, SEC_TONE_OFF) )
 		perror("FE_SET_TONE failed");
+	usleep(15*1000);
 	if ( ioctl(fdFrontend, FE_SET_VOLTAGE, ci%2 ? SEC_VOLTAGE_18 : SEC_VOLTAGE_13) )
 		perror("FE_SET_VOLTAGE failed");
-	usleep(15*1000);
+
 	fprintf( stderr, "DiSEqC: %02x %02x %02x %02x %02x %02x\n", switchCmd[ci].msg[0], switchCmd[ci].msg[1], switchCmd[ci].msg[2], switchCmd[ci].msg[3], switchCmd[ci].msg[4], switchCmd[ci].msg[5] );
-	if ( ioctl(fdFrontend, FE_DISEQC_SEND_MASTER_CMD, &switchCmd[ci]) )
-		perror("FE_DISEQC_SEND_MASTER_CMD failed");
+	for ( i=0; i<2; ++i ) {
+		usleep(15*1000);
+		if ( ioctl(fdFrontend, FE_DISEQC_SEND_MASTER_CMD, &switchCmd[ci]) )
+			perror("FE_DISEQC_SEND_MASTER_CMD failed");
+	}
 
 	QString msg;
 	if ( dvbDevice->lnb[switchPos].rotorType!=0 && chan->tp.source!=dvbDevice->lnb[switchPos].currentSource ) {
@@ -557,9 +586,12 @@ int DvbStream::setDiseqc( int switchPos, ChannelDesc *chan, int hiband, int &rot
 		}
 	}
 
-	usleep(15*1000);
-	if ( ioctl(fdFrontend, FE_DISEQC_SEND_BURST, (ci/4)%2 ? SEC_MINI_B : SEC_MINI_A) )
-		perror("FE_DISEQC_SEND_BURST failed");
+	for ( i=0; i<2; ++i ) {
+		usleep(15*1000);
+		if ( ioctl(fdFrontend, FE_DISEQC_SEND_BURST, (ci/4)%2 ? SEC_MINI_B : SEC_MINI_A) )
+			perror("FE_DISEQC_SEND_BURST failed");
+	}
+
 	usleep(15*1000);
 	if ( ioctl(fdFrontend, FE_SET_TONE, (ci/2)%2 ? SEC_TONE_ON : SEC_TONE_OFF) )
 		perror("FE_SET_TONE failed");
@@ -659,27 +691,24 @@ void DvbStream::rotorCommand( int cmd, int n1, int n2, int n3 )
 		{ { 0xe0, 0x10, 0x38, 0xF4, 0x00, 0x00 }, 4 }    //13 User
 	};
 
-	if ( ioctl( fdFrontend, FE_DISEQC_SEND_MASTER_CMD, &cmds[cmd] ) )
-		perror("Rotor : FE_DISEQC_SEND_MASTER_CMD failed");
+	int i;
+	for ( i=0; i<2; ++i ) {
+		usleep(15*1000);
+		if ( ioctl( fdFrontend, FE_DISEQC_SEND_MASTER_CMD, &cmds[cmd] ) )
+			perror("Rotor : FE_DISEQC_SEND_MASTER_CMD failed");
+	}
 }
 
 
 
-bool DvbStream::setPids( DVBout *o, int napid )
+bool DvbStream::setPids( DVBout *o )
 {
 	int i, dmx;
 	struct dmx_pes_filter_params pesFilterParams;
 	dmx_pes_type_t pestype = DMX_PES_OTHER;
 	QValueList<int> pidList;
 
-	if ( o->channel.vpid )
-		pidList.append( o->channel.vpid );
-	pidList.append( o->channel.apid[napid].pid );
-	if ( o->channel.subpid[0].pid )
-		pidList.append( o->channel.subpid[0].pid );
-
-	/*if ( (ndmx+pidList.count())>MAX_CHANNELS )
-		return false;*/
+	pidList = o->pidsList();
 
 	for ( i=0; i<(int)pidList.count(); i++ ) {
 		if ( ( dmx = open( demuxName.ascii(), O_RDWR)) < 0 ) {
@@ -899,19 +928,11 @@ void DvbStream::recordEnded( DVBout *o, RecTimer* t, bool kill )
 
 	if ( kill ) {
 		removePids( o );
+		if ( cam )
+			cam->stopService( &(o->channel) );
 		removeOut( o );
 		if ( out.count()==0 )
 			stop();
-		else if ( cam ) {
-			for ( i=0; i<(int)out.count(); i++ ) {
-				if ( out.at(i)->channel.fta ) {
-					i=-1;
-					break;
-				}
-			}
-			if ( i!=-1 )
-				cam->stop();
-		}
 	}
 	recordingState();
 	if ( t )
@@ -968,6 +989,8 @@ void DvbStream::stopBroadcast()
 	}
 	for ( i=0; i<(int)p.count(); i++ ) {
 		removePids( p.at(i) );
+		if ( cam )
+			cam->stopService( &(p.at(i)->channel) );
 		removeOut( p.at(i) );
 	}
 	if ( out.count()==0 )
@@ -984,7 +1007,7 @@ int DvbStream::canStartBroadcast( bool &live, ChannelDesc *chan )
 	for ( i=0; i<(int)out.count(); i++ ) {
 		if ( (chan->tp!=out.at(i)->channel.tp) && out.at(i)->hasRec() )
 			return ErrIsRecording;
-		if ( cam && out.at(i)->hasRec() && out.at(i)->channel.fta && chan->fta && out.at(i)->channel.sid!=chan->sid )
+		if ( chan->fta && cam && !cam->canPlay( chan ) )
 			return ErrCamUsed;
 		if ( out.at(i)->hasLive() && chan->tp!=out.at(i)->channel.tp )
 			live = true;
@@ -1033,7 +1056,7 @@ bool DvbStream::startBroadcast( QPtrList<ChannelDesc> *list, Ts2Rtp *rtp )
 			connect( o, SIGNAL(playDvb()), this, SLOT(receivePlayDvb()) );
 			connect( o, SIGNAL(shifting(bool)), this, SLOT(receiveShifting(bool)) );
 			out.append( o );
-			if ( !setPids( o, 0 ) ) {
+			if ( !setPids( o ) ) {
 				removePids( o );
 				removeOut( o );
 				o = 0;
@@ -1051,8 +1074,9 @@ bool DvbStream::startBroadcast( QPtrList<ChannelDesc> *list, Ts2Rtp *rtp )
 			}
 			else {
 				broadcastList.append( new ChannelDesc( *list->at(i) ) );
-				broadcastList.getLast()->ttpid = o->currentAudioPid(); // we use ttpid to store activeAudioPid
 				no++;
+				if ( list->at(i)->fta && cam )
+					cam->startService( list->at(i) );
 			}
 		}
 	}
@@ -1085,17 +1109,11 @@ int DvbStream::canStartTimer( bool &live, ChannelDesc *chan )
 			return ErrIsRecording;
 		if ( (o->channel.name==chan->name) && o->hasRec() )
 			return ErrIsRecording;
-		if ( (chan->tp==o->channel.tp) && o->hasRec() ) {
-			if ( chan->fta && cam && o->channel.fta )
-				return ErrCamUsed;
-		}
+		if ( chan->fta && cam && !cam->canPlay( chan ) )
+			return ErrCamUsed;
 		if ( o->hasLive() ) {
 			if ( chan->tp!=o->channel.tp )
 				live = true;
-			else if ( (o->channel.name==chan->name) && (o->currentAudioPid()!=0) )
-				live = true;
-			else if ( cam && chan->fta && o->channel.fta )
-				live = true;
 		}
 	}
 	return ret;
@@ -1103,7 +1121,7 @@ int DvbStream::canStartTimer( bool &live, ChannelDesc *chan )
 
 
 
-bool DvbStream::startTimer( ChannelDesc *chan, QString path, int maxsize, RecTimer *t )
+bool DvbStream::startTimer( ChannelDesc *chan, QString path, int maxsize, RecTimer *t, QString name )
 {
 	int i;
 	bool stop=false, newout=false;
@@ -1152,7 +1170,7 @@ bool DvbStream::startTimer( ChannelDesc *chan, QString path, int maxsize, RecTim
 		connect( o, SIGNAL(playDvb()), this, SLOT(receivePlayDvb()) );
 		connect( o, SIGNAL(shifting(bool)), this, SLOT(receiveShifting(bool)) );
 		out.append( o );
-		if ( !setPids( o, 0 ) ) {
+		if ( !setPids( o ) ) {
 			if ( stop )
 				stopFrontend();
 			removePids( o );
@@ -1162,10 +1180,13 @@ bool DvbStream::startTimer( ChannelDesc *chan, QString path, int maxsize, RecTim
 		newout = true;
 	}
 
-	QString name = path+t->name;
-	if ( t->mode )
-		name = name+"-"+t->begin.date().toString( Qt::ISODate );
-	if ( !o->goRec( t->filetype, name, maxsize, t ) ) {
+	if ( t->mode && !name.contains("%date") )
+		name+= "-%date";
+	name = name.replace( "%chan", chan->name ).replace("%date", t->begin.toString( "yyyyMMdd-hhmmss" ) ).replace("%name", t->name );
+	name = name.replace( "/", "_" ).replace( ">", "_" ).replace("<","_").replace(":","_").replace('"',"_").replace("\\","_").replace("|","_");
+	name = path+name;
+
+	if ( !o->goRec( name, maxsize, t ) ) {
 		if ( stop )
 			stopFrontend();
 		if ( newout ) {
@@ -1176,8 +1197,8 @@ bool DvbStream::startTimer( ChannelDesc *chan, QString path, int maxsize, RecTim
 	}
 
 	fprintf(stderr,"NOUT: %d\n", out.count() );
-	if ( chan->fta && cam && ( ((cam->running() && chan->sid!=cam->serviceId()) || !cam->running()) ) )
-		cam->restart( chan->sid );
+	if ( chan->fta && cam )
+		cam->startService( chan );
 
 	startReading();
 
@@ -1187,7 +1208,7 @@ bool DvbStream::startTimer( ChannelDesc *chan, QString path, int maxsize, RecTim
 
 
 
-int DvbStream::goLive( ChannelDesc *chan, const QString &pipeName, int &napid )
+int DvbStream::goLive( ChannelDesc *chan, const QString &pipeName )
 {
 	int i;
 	bool stop=false;
@@ -1198,10 +1219,8 @@ int DvbStream::goLive( ChannelDesc *chan, const QString &pipeName, int &napid )
 			return ErrIsRecording;
 		if ( (chan->tp!=out.at(i)->channel.tp) && out.at(i)->hasBroadcast() )
 			return ErrIsBroadcasting;
-		if ( (chan->tp==out.at(i)->channel.tp) && (out.at(i)->hasBroadcast() || out.at(i)->hasRec())) {
-			if ( chan->fta && cam && cam->running() && (cam->serviceId()!=chan->sid) )
-				return ErrCamUsed;
-		}
+		if ( chan->fta && cam && !cam->canPlay( chan ) )
+			return ErrCamUsed;
 		if ( out.at(i)->channel.name==chan->name )
 			o = out.at(i);
 	}
@@ -1227,7 +1246,7 @@ int DvbStream::goLive( ChannelDesc *chan, const QString &pipeName, int &napid )
 		connect( o, SIGNAL(playDvb()), this, SLOT(receivePlayDvb()) );
 		connect( o, SIGNAL(shifting(bool)), this, SLOT(receiveShifting(bool)) );
 		out.append( o );
-		if ( !setPids( o, napid ) ) {
+		if ( !setPids( o ) ) {
 			if ( stop )
 				stopFrontend();
 			removePids( o );
@@ -1236,17 +1255,16 @@ int DvbStream::goLive( ChannelDesc *chan, const QString &pipeName, int &napid )
 		}
 	}
 
-	if ( o->hasRec() ) {
-		napid = o->currentAudioPid();
+	if ( o->hasRec() )
 		i = ErrDontSwitchAudio;
-	}
-	else i = 0;
+	else
+		i = 0;
 
-	o->goLive( pipeName, napid );
+	o->goLive( pipeName );
 	fprintf(stderr,"NOUT: %d\n", out.count() );
 
-	if ( chan->fta && cam && !cam->running() )
-		cam->restart( chan->sid );
+	if ( chan->fta && cam  )
+		cam->startService( chan );
 	startReading();
 
 	return i;
@@ -1290,17 +1308,12 @@ void DvbStream::stopLive( ChannelDesc *chan )
 	}
 	for ( i=0; i<(int)p.count(); i++ ) {
 		removePids( p.at(i) );
+		if ( cam )
+			cam->stopService( &(p.at(i)->channel) );
 		removeOut( p.at(i) );
 	}
 	fprintf(stderr,"Live stopped\n");
-	if ( cam ) {
-		for ( i=0; i<(int)out.count(); i++ ) {
-			if ( out.at(i)->channel.fta )
-				camUsed = true;
-		}
-	}
-	if ( cam && !camUsed )
-		cam->stop();
+
 	if ( out.count()==0 && chan->tp!=currentTransponder )
 		stop();
 }
@@ -1312,7 +1325,7 @@ void DvbStream::startReading()
 	if ( !isRunning ) {
 		isRunning = true;
 		start();
-		dvbEvents->go( currentTransponder.source, true );
+		dvbEvents->go( currentTransponder.source, currentTransponder.freq, true );
 	}
 }
 
@@ -1321,13 +1334,11 @@ void DvbStream::startReading()
 void DvbStream::stop()
 {
 	isRunning = false;
-	if ( !wait(5000) ) {
+	if ( !wait(10000) ) {
 		terminate();
 		wait();
 		fprintf(stderr,"dvbstream::run() terminated\n");
 	}
-	if ( cam )
-		cam->stop();
 	dvbEvents->stop();
 	stopFrontend();
 }
@@ -1364,7 +1375,6 @@ DvbStream::~DvbStream()
 {
 	stop();
 	if ( cam ) {
-		cam->stop();
 		delete cam;
 	}
 	delete dvbEvents;
