@@ -32,11 +32,10 @@
 
 
 
-KRecord::KRecord( QStringList chanList, QPtrList<RecTimer> *t, QWidget *parent, QSize size, int format ) : QDialog( parent )
+KRecord::KRecord( QStringList chanList, QPtrList<RecTimer> *t, QWidget *parent, QSize size ) : QDialog( parent )
 {
 	timers = t;
 	channelsList = chanList;
-	prefFormat = format;
 
 	QVBoxLayout *vb = new QVBoxLayout( 0, 0, 6 );
 	QLabel *lab = new QLabel( i18n("Timers list:"), this );
@@ -126,7 +125,7 @@ void KRecord::refresh()
 		else s = "";
 		s = s+ti->name;*/
 		s = ti->name;
-		itt = new KListViewItem( (QListView*)list, "", s, ti->channel, ti->begin.toString("hh:mm dd/MM/yy"), ti->duration.toString("hh:mm") );
+		itt = new KListViewItem( (QListView*)list, "", s, ti->channel, KGlobal::locale()->formatDateTime( ti->begin ), ti->duration.toString("hh:mm") );
 		if ( ti->mode )
 			itt->setPixmap( 0, yesRepeat );
 		if ( ti->running )
@@ -144,7 +143,6 @@ void KRecord::newTimer()
 	RecTimer *t = new RecTimer();
 
 	t->running = 0;
-	t->filetype = prefFormat;
 	t->mode = 0;
 
 	KTimerEditor dlg( true, channelsList, *t, this );
@@ -152,7 +150,7 @@ void KRecord::newTimer()
 	if ( ret==KTimerEditor::Accepted ) {
 		*t = dlg.timer;
 		after = where( t, true );
-		itt = new KListViewItem( (QListView*)list, "", t->name, t->channel, t->begin.toString("hh:mm dd/MM/yy"), t->duration.toString("hh:mm") );
+		itt = new KListViewItem( (QListView*)list, "", t->name, t->channel, KGlobal::locale()->formatDateTime( t->begin ), t->duration.toString("hh:mm") );
 		if ( t->mode )
 			itt->setPixmap( 0, yesRepeat );
 		if ( t->running )
