@@ -146,12 +146,15 @@ void InputManager::showMe( KaffeineInput *p )
 {
 	int id=0;
 	int p_id=0;
+	QWidget *widg;
 
 	for ( QPtrListIterator<InputPlugin> it(plugs); it.current(); ++it ) {
 		if ( it.current()->plug->inputMainWidget()==currentMainWidget )
 			id = it.current()->id;
-		if ( it.current()->plug==p )
+		if ( it.current()->plug==p ) {
+			widg = it.current()->plug->inputMainWidget();
 			p_id = it.current()->id;
+		}
 	}
 	if ( currentMainWidget==playerWidget )
 		id = 2;
@@ -159,9 +162,13 @@ void InputManager::showMe( KaffeineInput *p )
 		id = 1;
 
 	if ( id && p_id && (id!=p_id) ) {
-		mtBar->setTab( id, false );
-		mtBar->setTab( p_id, true );
-		show( p_id );
+		if ( mtBar->isHidden() )
+			currentMainWidget = widg;
+		else {
+			mtBar->setTab( id, false );
+			mtBar->setTab( p_id, true );
+			show( p_id );
+		}
 	}
 }
 
@@ -254,6 +261,19 @@ void InputManager::addConfigKeys( KKeyDialog* kd )
 
 
 
+bool InputManager::isPlaylistCurrent()
+{
+	for ( QPtrListIterator<InputPlugin> it(plugs); it.current(); ++it ) {
+		if ( it.current()->name == i18n("Playlist") ) {
+			if ( it.current()->plug==currentPlugin )
+				return true;
+		}
+	}
+	return false;
+}
+
+
+
 void InputManager::playCurrentTrack()
 {
 	MRL mrl;
@@ -558,9 +578,11 @@ void PlayerContainer::dropEvent(QDropEvent* dev)
 		{
 			for (uint i=0; i < urls.count() ;i++)
 			{
-				KURL url(QUriDrag::unicodeUriToUri(urls[i]));
-				newurls << url.path(-1);
-				kdDebug() << "Kaffeine: Dropped " << url.path() << endl;
+				//KURL url(QUriDrag::unicodeUriToUri(urls[i]));
+				//newurls << url.path(-1);
+				//kdDebug() << "Kaffeine: Dropped " << url.path() << endl;
+				newurls << urls[i];
+				kdDebug() << "Kaffeine: Dropped " << urls[i] << endl;
 			}
 			emit signalURLDropEvent(newurls);
 		}
